-module(task_limiter).
-compile([no_auto_import, nowarn_unused_vars, nowarn_unused_function, nowarn_nomatch]).

-export([async_await/3]).

-if(?OTP_RELEASE >= 27).
-define(MODULEDOC(Str), -moduledoc(Str)).
-define(DOC(Str), -doc(Str)).
-else.
-define(MODULEDOC(Str), -compile([])).
-define(DOC(Str), -compile([])).
-endif.

-file("src/task_limiter.gleam", 22).
-spec run_helper(
    list(gleam@otp@task:task(HSF)),
    list(fun(() -> HSF)),
    list({ok, HSF} | {error, gleam@otp@task:await_error()}),
    integer()
) -> list({ok, HSF} | {error, gleam@otp@task:await_error()}).
run_helper(Running, Waiting, Done, Timeout) ->
    case Running of
        [] ->
            Done;

        _ ->
            Results = begin
                _pipe = gleam@otp@task:try_await_all(Running, Timeout),
                gleam@list:zip(_pipe, Running)
            end,
            {Finished, Working} = begin
                _pipe@1 = Results,
                gleam@list:partition(_pipe@1, fun(Tup) -> case Tup of
                            {{error, timeout}, _} ->
                                false;

                            _ ->
                                true
                        end end)
            end,
            Finished_result = begin
                _pipe@2 = Finished,
                gleam@list:map(_pipe@2, fun gleam@pair:first/1)
            end,
            Working_tasks = begin
                _pipe@3 = Working,
                gleam@list:map(_pipe@3, fun gleam@pair:second/1)
            end,
            N_to_add = erlang:length(Finished),
            {To_add, New_waiting} = gleam@list:split(Waiting, N_to_add),
            run_helper(
                lists:append(
                    begin
                        _pipe@4 = To_add,
                        gleam@list:map(_pipe@4, fun gleam@otp@task:async/1)
                    end,
                    Working_tasks
                ),
                New_waiting,
                lists:append(Finished_result, Done),
                Timeout
            )
    end.

-file("src/task_limiter.gleam", 11).
?DOC(
    " Run a set of functions asynchronously as OTP task num at a time, then\n"
    " waits for the value computed by the tasks to return. Every interval\n"
    " milliseconds the running jobs are checked to see if they have completed\n"
    " and new jobs from the queue are started.\n"
    "\n"
    " **Note**: The values are returned in an arbitrary order.\n"
).
-spec async_await(list(fun(() -> HSA)), integer(), integer()) -> list({ok, HSA} |
    {error, gleam@otp@task:await_error()}).
async_await(Work, Num, Interval) ->
    {Run_group, Wait_group} = gleam@list:split(Work, Num),
    Running = begin
        _pipe = Run_group,
        gleam@list:map(_pipe, fun gleam@otp@task:async/1)
    end,
    run_helper(Running, Wait_group, [], Interval).
